local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")
local faction = UnitFactionGroup("player")

function Capitalist:UpdateRestockStack(profession)
	self:DebugEntry("UpdateRestockStack")

	self.inTradeskill = nil

	if (not self.restock) then
		self.restock = {}
	end

	--print(profession)
	self.restock[profession] = nil

	local oRealm = self:GetProfile_Realm_Faction(realm, faction)
	for crafter, crafterData in pairs(oRealm) do
		local professionData = crafterData[profession]

		-- don't revisit existing products already tallied
		-- only process if crafter has the profession
		if ((not self.restock[profession]) and professionData) then
			self.restock[profession] = {}

			for skillName, skillData in pairs(professionData) do
				local productCount = self:ProductGetCount(profession, skillName)
				if ((productCount > 0) and
					self:ProductIsEnabled(profession, skillName) and
					self:ProductIsVisible(profession, skillName)) then

					local productName = self:ProductNameFromSkillName(profession, skillName)
					--print(productName)

					-- REVIEW: This was causing the (R) for JC to not report a missing-from-AH gem because a non-JC character had one in inventory
					--         (Discovered when testing initial implementation of (unchecked) crossScan.)
					if (false) then
						-- search AH and all inventories on this realm for product
						for player, t in pairs(self.db.profile.storage[realm]) do
							if (t.AH[productName]) then
								productCount = productCount - t.AH[productName]
							end
							-- REVIEW: is 'bank' missing here or intended/(already included)?
							if (t.inventory[productName]) then
								productCount = productCount - t.inventory[productName]
							end
						end
					end
					-- REVIEW: (continued)
					if (self.db.profile.storage[realm][character].AH[productName]) then
						productCount = productCount - self.db.profile.storage[realm][character].AH[productName]
					end
					if (self.db.profile.storage[realm][character].inventory[productName]) then
						productCount = productCount - self.db.profile.storage[realm][character].inventory[productName]
					end
					-- REVIEW: (end)

			
					if (productCount > 0) then
						self.restock[profession][skillName] = productCount
						--print(skillName)
					end
				end
			end
	
			self.requiredComponents = {}

			for skillName, count in pairs(self.restock[profession]) do
				local product = self.db.profile.professions[profession][skillName]

				for componentName, componentCount in pairs(product.components) do
					if (self.requiredComponents[componentName] == nil) then
						self.requiredComponents[componentName] = componentCount * count
					else
						self.requiredComponents[componentName] = componentCount * count + self.requiredComponents[componentName]
					end
				end

				if (profession == L["Enchanting"]) then
					local componentName = L["Enchanting Vellum"]
					if (self.requiredComponents[componentName] == nil) then
						self.requiredComponents[componentName] = count;
					else
						self.requiredComponents[componentName] = count + self.requiredComponents[componentName];
					end
				end
			end
		end
	end

	self:DebugExit("UpdateRestockStack")
end

function Capitalist:ShowRestockStack(profession)
	self:DebugEntry("ShowRestockStack")

	self:UpdateRestockStack(profession)

	local list = {}
	local numRestock = 0
	local components = {}
	for k, v in pairs(self.requiredComponents) do
		components[k] = v
	end
	
	for skillName, count in pairs(self.restock[profession]) do
		list[skillName] = count
		numRestock = numRestock + count
	end
	
	-- search for the required components in our inventory
	for bag = 0, 4 do
		for slot = 0, GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if (link) then
				local name = link:match("%[(.*)%]")
				if (components[name]) then
					local _, count = GetContainerItemInfo(bag, slot)
					components[name] = components[name] - count
				end
			end
		end
	end
	
	-- output the restock stack
	if (numRestock == 0) then
		self:Print(L["Nothing to restock!"])
		return
	else
		if (numRestock == 1) then
			self:Print(L["Need to restock 1 product:"])
		else
			self:Print(string.format(L["Need to restock %d products:"], numRestock))
		end
		for skillName, count in pairs(list) do

			local skillText = skillName
			if (self.db.profile.professions and
				self.db.profile.professions[profession] and
				self.db.profile.professions[profession][skillName] and
				self.db.profile.professions[profession][skillName].itemLink) then
				skillText = self:SkillTextFromSkillName(profession, skillName)
			end

			ChatFrame1:AddMessage("  "..count.."x "..skillText)
		end
	end
	
	-- output any components missing
	list = {}
	local numMissing = 0
	for componentName, componentCount in pairs(components) do
		if (componentCount > 0) then
			list[componentName] = componentCount
			numMissing = numMissing + componentCount
		end
	end
	
	if (numMissing > 0) then
		self:Print(L["Missing components:"])
		for item, count in pairs(list) do
			local itemText = item
			if (self.db.profile.components and
				self.db.profile.components[item] and
				self.db.profile.components[item].link) then
				itemText = self.db.profile.components[item].link
			end
			ChatFrame1:AddMessage("  "..count.."x "..itemText)
		end

		-- search all inventories on this realm for components
		local outputtedHeaderAltInventory
		for componentName, count in pairs(list) do
			local s
			for player, t in pairs(self.db.profile.storage[realm]) do
				if (character ~= player) then
					local c = t.inventory[componentName]
					if (c and (c > 0)) then
						if (s) then
							s = s..", x"..c.." "..player
						else
							local componentText = componentName
							if (self.db.profile.components and
								self.db.profile.components[componentName] and
								self.db.profile.components[componentName].link) then
								componentText = self.db.profile.components[componentName].link
							end
							s = "  "..componentText.."   x"..c.." "..player
						end
					end
				end
			end
			if (s and not outputtedHeaderAltInventory) then
				self:Print(L["Components in alt inventories:"])
				outputtedHeaderAltInventory = true
			end
			if (s) then
				ChatFrame1:AddMessage(s)
			end
		end

		-- search all banks on this realm for components
		local outputtedHeaderBank
		for componentName, count in pairs(list) do
			local s
			for player, t in pairs(self.db.profile.storage[realm]) do
				local c = t.bank[componentName]
				if (c and (c > 0)) then
					if (s) then
						s = s..", x"..c.." "..player
					else
						local componentText = componentName
						if (self.db.profile.components and
							self.db.profile.components[componentName] and
							self.db.profile.components[componentName].link) then
							componentText = self.db.profile.components[componentName].link
						end
						s = "  "..componentText.."   x"..c.." "..player
					end
				end
			end
			if (s and not outputtedHeaderBank) then
				self:Print(L["Components in bank:"])
				outputtedHeaderBank = true
			end
			if (s) then
				ChatFrame1:AddMessage(s)
			end
		end
	else
		self:Print(L["Everything available!"])
	end

	self:DebugExit("ShowRestockStack")
end

function Capitalist:Restock(profession)
	self:DebugEntry("Restock")

	if (not self.inTradeskill) then
		C_TradeSkillUI.ClearInventorySlotFilter()
		C_TradeSkillUI.ClearRecipeCategoryFilter()
		C_TradeSkillUI.ClearRecipeSourceTypeFilter()
	
		if (self.restock) then
			for k, v in pairs(self.restock[profession]) do
				if (self:TryRestock(profession, k)) then
					return
				end
			end
		end
	
		self:Print(L["Could not restock anything!"])
	end

	self:DebugExit("Restock")
end

function Capitalist:TryRestock(profession, skillName)
	self:DebugEntry("TryRestock")
	self:DebugMisc(profession)
	self:DebugMisc(skillName)

	local bag
	local slot

	local isEnchanting = (profession == L["Enchanting"])

	-- REVIEW: removable?
	if (isEnchanting) then
	--print("IsEnchanting")
		local vellum = L["Enchanting Vellum"];

		for i = 0, 4 do
			for j = 1, GetContainerNumSlots(i) do
				local link = GetContainerItemLink(i, j)
				if (link) then
					local name = link:match("%[(.*)%]")
					if (name == vellum) then
						--print("FoundVellum")
						bag = i
						slot = j
					end
				end
			end
		end

		if (not bag) then
			--print("MissingVellum")
			return false
		end
	end

	local itemLink = self.db.profile.professions[profession][skillName].itemLink
	--print(itemLink)

	local allRecipeIDs = C_TradeSkillUI.GetAllRecipeIDs()
	--for i = 1, GetNumTradeSkills() do
	for i, recipeID in pairs(allRecipeIDs) do
		local skill = C_TradeSkillUI.GetRecipeInfo(recipeID)
		if (skill.learned) then
			--local _, _, numAvailable = GetTradeSkillInfo(i)
			local numAvailable = skill.numAvailable
			--print(numAvailable)
			--local itemLinkFound = GetTradeSkillItemLink(i)
			local itemLinkFound = C_TradeSkillUI.GetRecipeItemLink(skill.recipeID)
			--print(itemLinkFound)

			if ((itemLinkFound == itemLink) and (numAvailable > 0)) then
				self:DebugMisc(product)
				--print(product)

				-- REVIEW: Opportunity for multi
				--DoTradeSkill(i)
				C_TradeSkillUI.CraftRecipe(recipeID)
				if (isEnchanting) then
					UseContainerItem(bag, slot)
				end
				self.inTradeskill = profession
				return self:DebugExit("TryRestock", true)
			end
		end
	end

	return self:DebugExit("TryRestock", false)
end
